# Overview of HLT jet reco configuration modules

## GenerateJetChainDefs.py

Called by the menu code in `[TriggerMenuMT/python/HLTMenuConfig/Menu/GenerateMenuMT.py](https://acode-browser1.usatlas.bnl.gov/lxr/source/athena/Trigger/TriggerCommon/TriggerMenuMT/python/HLTMenuConfig/Menu/GenerateMenuMT.py)` to translate the HLT chain item into a concrete algorithm sequence.

The menu code creates a chain dictionary from the chain name, of which the jet parts are given to `GenerateJetChainDefs.generateChainConfigs` to be interpreted by `JetChainConfiguration`.

## JetChainConfiguration.py

Defines the `JetChainConfiguration` object responsible for interpreting the chain dictionary and building a `Chain` object that is returned to the menu.

This extracts the reco configuration from the jet chain dictionary, using it to generate a `MenuSequence` that forms the jet `ChainStep`. It may be desirable to implement multiple `ChainSteps` for filtering purposes in the future, mainly to allow fast reco and filtering before slower reco is executed.

`JetChainConfiguration` internally calls `JetMenuSequences.jetMenuSequence()` to produce the `MenuSequence` item, which contains a reco sequence followed by a hypo selection.

## JetMenuSequences.py

The jet chains are currently made up of only a single step, which runs reco+hypo. The menu sequence is constructed from:
* a concrete reco sequence, generated by `JetRecoSequences.jetAthSequence()`;
* the `InputMakerAlg` for this sequence (largely irrelevant as jets work in FullScan rather than in EventViews);
* a hypo alg (one instance per jet collection i.e. reco config); and
*  a hypo tool generator function that will define the selection based on the `chainDict` contents (configuration defined in `TrigHLTJetHypo`).

The reco sequence and hypo alg names are suffixed with a string summarising the reco options.

## JetRecoSequences.py

This module provides the `jetAthSequence()` and `jetRecoSequence()` functions that define the reconstruction sequence for any given reconstruction configuration.

The `jetAthSequence` holds the full reconstruction as well as an `InputMakerAlg` which for FullScan jet triggers (AFAIK) mostly serves to communicate L1 seed information to determine which hypo tools should be run.

The reconstruction sequence is determined from the contents of the reco information in the `chainDict`, and will contain some subset of:
* Calo reco sequence: cell unpacking and topoclustering -- one instance shared between all jet chains
* Constituent modifications [optional] -- pileup suppression on topoclusters or corrections to PFlow four-vectors
* PseudoJetGetters & algs -- conversion of the ATLAS EDM into `fastjet` EDM
* JetAlgorithm -- holds the jet finder tools and any modifiers e.g. calibration
The reco sequence may be nested further in the case of reclustering or trimming workflows, in which case the "basic" jet reco from clusters is embedded in a second sequence, which continues by running the second step reconstruction.
Configuration of the Athena components is handled by the `Reconstruction/Jet/JetRecConfig` package.